import 'reflect-metadata';
import { TypedConstructor } from '../common/TypedConstructor';
export declare type CreateEmbedTypeFunctionReturn<T> = TypedConstructor<T> | Function;
export declare type CreateEmbedTypeFunction<T> = (doc: any) => CreateEmbedTypeFunctionReturn<T>;
/**
 * Many embedded documents can be a normal array, map, or key/value object.
 */
export declare enum EmbedManyAs {
    MAP = "map",
    ARRAY = "array",
    OBJECT = "object",
}
/**
 * Whether or not the class is a document or embedded document.
 */
export declare enum DocumentMetadataType {
    DOCUMENT = "DOCUMENT",
    EMBEDDED_DOCUMENT = "EMBEDDED_DOCUMENT",
}
/**
 * Whether it's a normal field or embedded document(s).
 */
export declare enum DocumentMetadataFieldType {
    FIELD = "FIELD",
    EMBED_ONE = "EMBED_ONE",
    EMBED_ARRAY = "EMBED_ARRAY",
    EMBED_MAP = "EMBED_MAP",
    EMBED_OBJECT = "EMBED_OBJECT",
}
/**
 * DocumentMetadata contains all the information about classes that get mapped to MongoDB documents.
 */
export interface DocumentMetadata {
    /**
     * The base document class.
     */
    target: any;
    /**
     * Flag whether the metadata has been processed or not.
     */
    isProcessed: boolean;
    /**
     * Is this a Document or Embedded Document?
     */
    type?: DocumentMetadataType;
    /**
     * The property to document field mappings.
     */
    fields?: {
        [name: string]: DocumentMetadataField;
    };
    /**
     * If the class extends another.
     */
    parent?: any;
}
/**
 * The document or embedded document field options.
 */
export interface DocumentMetadataField {
    /**
     * Whether it's a normal field or embedded document(s).
     */
    type: DocumentMetadataFieldType;
    /**
     * The reflect "design:type" for later use.
     */
    designType: any;
    /**
     * This option is required on embedded field types.
     */
    createEmbeddedInstance?: <T>(doc: any) => T;
}
/**
 * Registers the given class as a document or embedded document.
 */
export declare function defineDocumentMetadata<T>(DocumentClass: TypedConstructor<T> | Function, type?: DocumentMetadataType): void;
/**
 * Define's a mapped field for a document or embedded document.
 */
export declare function defineDocumentMetadataField<T>(DocumentClass: TypedConstructor<T> | Function, name: string, opts?: Partial<DocumentMetadataField>): void;
/**
 * Define's a mapped field for a document or embedded document.
 */
export declare function defineDocumentMetadataEmbeddedField<T>(DocumentClass: TypedConstructor<T> | Function, name: string, createEmbedType: <E>(doc: any) => CreateEmbedTypeFunctionReturn<E>, embedManyAs?: EmbedManyAs): void;
/**
 * Returns true if the given class is a document.
 */
export declare function isDocument<T>(EmbeddedDocumentClass: TypedConstructor<T> | Function): boolean;
/**
 * Returns true if the given class is an embedded document.
 */
export declare function isEmbeddedDocument<T>(EmbeddedDocumentClass: TypedConstructor<T> | Function): boolean;
/**
 * Gets the document metadata for documents or embedded documents
 */
export declare function getDocumentMetadata<T>(DocumentClass: TypedConstructor<T> | Function): DocumentMetadata;
/**
 * Merges parent's fields into the upper-most document.
 */
export declare function processAndValidateMetadata(meta?: DocumentMetadata): void;
